# Copyright 2022 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

class DimensionSetExceededError(Exception):
    def __init__(self, message: str) -> None:
        # Call the base class constructor with the parameters it needs
        super().__init__(message)


class InvalidDimensionError(Exception):
    def __init__(self, message: str) -> None:
        # Call the base class constructor with the parameters it needs
        super().__init__(message)


class InvalidMetricError(Exception):
    def __init__(self, message: str) -> None:
        # Call the base class constructor with the parameters it needs
        super().__init__(message)


class InvalidNamespaceError(Exception):
    def __init__(self, message: str) -> None:
        # Call the base class constructor with the parameters it needs
        super().__init__(message)
